@@ -138,13 +138,13 @@ struct _garray
 
 static t_pd *garray_arraytemplatecanvas;
 static char garray_arraytemplatefile[] = "\
-#N canvas 0 0 458 153 10;\n\
+canvas 0 0 458 153 10;\n\
 #X obj 43 31 struct _float_array array z float float style\n\
 float linewidth float color;\n\
 #X obj 43 70 plot z color linewidth 0 0 1 style;\n\
 ";
 static char garray_floattemplatefile[] = "\
-#N canvas 0 0 458 153 10;\n\
+canvas 0 0 458 153 10;\n\
 #X obj 39 26 struct float float y;\n\
 ";
 
@@ -160,12 +160,12 @@ void garray_init( void)
     
     glob_setfilename(0, gensym("_float"), gensym("."));
     binbuf_text(b, garray_floattemplatefile, strlen(garray_floattemplatefile));
-    binbuf_eval(b, 0, 0, 0);
+    binbuf_eval(b, &pd_canvasmaker, 0, 0);
     vmess(s__X.s_thing, gensym("pop"), "i", 0);
     
     glob_setfilename(0, gensym("_float_array"), gensym("."));
     binbuf_text(b, garray_arraytemplatefile, strlen(garray_arraytemplatefile));
-    binbuf_eval(b, 0, 0, 0);
+    binbuf_eval(b, &pd_canvasmaker, 0, 0);
     garray_arraytemplatecanvas = s__X.s_thing;
     vmess(s__X.s_thing, gensym("pop"), "i", 0);
 
@@ -280,7 +280,7 @@ t_garray *graph_array(t_glist *gl, t_symbol *s, t_symbol *templateargsym,
     t_floatarg fsize, t_floatarg fflags)
 {
     int n = fsize, i, zz, nwords, zonset, ztype, saveit;
-    t_symbol *zarraytype;
+    t_symbol *zarraytype, *asym = gensym("#A");
     t_garray *x;
     t_pd *x2;
     t_template *template, *ztemplate;
@@ -332,10 +332,15 @@ t_garray *graph_array(t_glist *gl, t_symbol *s, t_symbol *templateargsym,
         style, 1);
     template_setfloat(template, gensym("linewidth"), x->x_scalar->sc_vec, 
         ((style == PLOTSTYLE_POINTS) ? 2 : 1), 1);
-    if (x2 = pd_findbyclass(gensym("#A"), garray_class))
-        pd_unbind(x2, gensym("#A"));
 
-    pd_bind(&x->x_gobj.g_pd, gensym("#A"));
+           /* bashily unbind #A -- this would create garbage if #A were
+           multiply bound but we believe in this context it's at most
+           bound to whichever textobj or array was created most recently */
+    asym->s_thing = 0;
+        /* and now bind #A to us to receive following messages in the
+        saved file or copy buffer */
+    pd_bind(&x->x_gobj.g_pd, asym); 
+
     garray_redraw(x);
     return (x);
 }
@@ -400,7 +405,10 @@ void garray_arraydialog(t_garray *x, t_symbol *name, t_floatarg fsize,
             gensym("style"), x->x_scalar->sc_vec, 1);
     if (deleteit != 0)
     {
+        int wasused = x->x_usedindsp;
         glist_delete(x->x_glist, &x->x_gobj);
+        if (wasused)
+            canvas_update_dsp();
     }
     else
     {
@@ -559,7 +567,7 @@ static void garray_free(t_garray *x)
     /* } jsarlo */
     gfxstub_deleteforkey(x);
     pd_unbind(&x->x_gobj.g_pd, x->x_realname);
-        /* LATER find a way to get #A unbound earlier (at end of load?) */
+        /* just in case we're still bound to #A from loading... */
     while (x2 = pd_findbyclass(gensym("#A"), garray_class))
         pd_unbind(x2, gensym("#A"));
     pd_free(&x->x_scalar->sc_gobj.g_pd);
@@ -1156,11 +1164,14 @@ int garray_getfloatarray(t_garray *x, int *size, t_float **vec)
 {
     if (sizeof(t_word) != sizeof(t_float))
     {
-        static int warned;
-        if (!warned)
-            post(
- "warning: extern using garray_getfloatarray() won't work in 64-bit version");
-        warned = 1;
+        t_symbol *patchname;
+        if (x->x_glist->gl_owner)
+            patchname = x->x_glist->gl_owner->gl_name;
+        else
+            patchname = x->x_glist->gl_name;
+        error("An operation on the array '%s' in the patch '%s'",
+              x->x_name->s_name, patchname->s_name);
+        error("failed since it uses garray_getfloatarray while running 64-bit!");
     }
     return (garray_getfloatwords(x, size, (t_word **)vec));
 }
@@ -1435,8 +1446,7 @@ static void garray_write(t_garray *x, t_symbol *filename)
     }
     canvas_makefilename(glist_getcanvas(x->x_glist), filename->s_name,
         buf, MAXPDSTRING);
-    sys_bashfilename(buf, buf);
-    if (!(fd = fopen(buf, "w")))
+    if (!(fd = sys_fopen(buf, "w")))
     {
         error("%s: can't create", buf);
         return;
