@@ -95,8 +95,9 @@ void glist_delete(t_glist *x, t_gobj *y)
             }
             else
             {
-                text_eraseborder(&gl->gl_obj, x,
-                    rtext_gettag(glist_findrtext(x, &gl->gl_obj)));
+                if (glist_isvisible(x))
+                    text_eraseborder(&gl->gl_obj, x,
+                        rtext_gettag(glist_findrtext(x, &gl->gl_obj)));
             }
         }
     }
@@ -107,7 +108,9 @@ void glist_delete(t_glist *x, t_gobj *y)
             glist_getcanvas(x)->gl_name)), 2);
     gobj_delete(y, x);
     if (glist_isvisible(canvas))
+    {
         gobj_vis(y, x, 0);
+    }
     if (x->gl_editor && (ob = pd_checkobject(&y->g_pd)))
         rtext_new(x, ob);
     if (x->gl_list == y) x->gl_list = y->g_next;
@@ -271,19 +274,6 @@ void glist_sort(t_glist *x)
         x->gl_list = glist_dosort(x, x->gl_list, nitems);
 }
 
-void glist_cleanup(t_glist *x)
-{
-    freebytes(x->gl_xlabel, x->gl_nxlabels * sizeof(*(x->gl_xlabel)));
-    freebytes(x->gl_ylabel, x->gl_nylabels * sizeof(*(x->gl_ylabel)));
-    gstub_cutoff(x->gl_stub);
-}
-
-void glist_free(t_glist *x)
-{
-    glist_cleanup(x);
-    freebytes(x, sizeof(*x));
-}
-
 /* --------------- inlets and outlets  ----------- */
 
 
@@ -984,9 +974,12 @@ static void graph_delete(t_gobj *z, t_glist *glist)
 {
     t_glist *x = (t_glist *)z;
     t_gobj *y;
-    text_widgetbehavior.w_deletefn(z, glist);
     while (y = x->gl_list)
         glist_delete(x, y);
+    if (glist_isvisible(x))
+    {
+        text_widgetbehavior.w_deletefn(z, glist);
+    }
 }
 
 static t_float graph_lastxpix, graph_lastypix;
