@@ -168,6 +168,9 @@ void vinlet_dspprolog(struct _vinlet *x, t_signal **parentsigs,
     int downsample, int upsample,  int reblock, int switched)
 {
     t_signal *insig, *outsig;
+        /* no buffer means we're not a signal inlet */
+    if (!x->x_buf)
+        return;
     x->x_updown.downsample = downsample;
     x->x_updown.upsample   = upsample;
 
@@ -220,8 +223,10 @@ void vinlet_dspprolog(struct _vinlet *x, t_signal **parentsigs,
                     dsp_add(vinlet_doprolog, 3, x, insig->s_vec,
                         re_parentvecsize);
             else {
+              int method = (x->x_updown.method == 3?
+                (pd_compatibilitylevel < 044 ? 0 : 1) : x->x_updown.method);
               resamplefrom_dsp(&x->x_updown, insig->s_vec, parentvecsize,
-                re_parentvecsize, x->x_updown.method);
+                re_parentvecsize, method);
               dsp_add(vinlet_doprolog, 3, x, x->x_updown.s_vec,
                 re_parentvecsize);
         }
@@ -260,9 +265,13 @@ static void *vinlet_newsig(t_symbol *s)
      *
      * up till now we provide several upsampling methods and 1 single downsampling method (no filtering !)
      */
-    if (s == gensym("hold"))x->x_updown.method=1;     /* up: sample and hold */
-    else if (s == gensym("lin"))x->x_updown.method=2; /* up: linear interpolation */
-    else x->x_updown.method=0;                        /* up: zero-padding */
+    if (s == gensym("hold"))
+        x->x_updown.method=1;       /* up: sample and hold */
+    else if (s == gensym("lin") || s == gensym("linear"))
+        x->x_updown.method=2;       /* up: linear interpolation */
+    else if (s == gensym("pad"))
+        x->x_updown.method=0;       /* up: zero-padding */
+    else x->x_updown.method=3;      /* sample/hold unless version<0.44 */
 
     return (x);
 }
@@ -292,10 +301,10 @@ typedef struct _voutlet
     t_canvas *x_canvas;
     t_outlet *x_parentoutlet;
     int x_bufsize;
-    t_float *x_buf;         /* signal buffer; zero if not a signal */
-    t_float *x_endbuf;
-    t_float *x_empty;       /* next to read out of buffer in epilog code */
-    t_float *x_write;       /* next to write in to buffer */
+    t_sample *x_buf;         /* signal buffer; zero if not a signal */
+    t_sample *x_endbuf;
+    t_sample *x_empty;       /* next to read out of buffer in epilog code */
+    t_sample *x_write;       /* next to write in to buffer */
     int x_hop;              /* hopsize */
         /* vice versa from the inlet, if we don't block, this holds the
         parent's outlet signal, valid between the prolog and the dsp setup
@@ -373,7 +382,7 @@ t_int *voutlet_perform(t_int *w)
     t_voutlet *x = (t_voutlet *)(w[1]);
     t_float *in = (t_float *)(w[2]);
     int n = (int)(w[3]);
-    t_float *out = x->x_write, *outwas = out;
+    t_sample *out = x->x_write, *outwas = out;
 #if 0
     if (tot < 5) post("-in %lx out %lx n %d", in, out, n);
     if (tot < 5) post("-buf %lx endbuf %lx", x->x_buf, x->x_endbuf);
@@ -393,10 +402,10 @@ t_int *voutlet_perform(t_int *w)
 static t_int *voutlet_doepilog(t_int *w)
 {
     t_voutlet *x = (t_voutlet *)(w[1]);
-    t_float *out = (t_float *)(w[2]);
+    t_sample *out = (t_sample *)(w[2]);
 
     int n = (int)(w[3]);
-    t_float *in = x->x_empty;
+    t_sample *in = x->x_empty;
     if (x->x_updown.downsample != x->x_updown.upsample)
         out = x->x_updown.s_vec;
 
@@ -413,8 +422,8 @@ static t_int *voutlet_doepilog_resampling(t_int *w)
 {
     t_voutlet *x = (t_voutlet *)(w[1]);
     int n = (int)(w[2]);
-    t_float *in  = x->x_empty;
-    t_float *out = x->x_updown.s_vec;
+    t_sample *in  = x->x_empty;
+    t_sample *out = x->x_updown.s_vec;
 
 #if 0
     if (tot < 5) post("outlet in %lx out %lx n %lx", in, out, n), tot++;
@@ -435,6 +444,9 @@ void voutlet_dspprolog(struct _voutlet *x, t_signal **parentsigs,
     int myvecsize, int calcsize, int phase, int period, int frequency,
     int downsample, int upsample, int reblock, int switched)
 {
+        /* no buffer means we're not a signal outlet */
+    if (!x->x_buf)
+        return;
     x->x_updown.downsample=downsample;
     x->x_updown.upsample=upsample;
     x->x_justcopyout = (switched && !reblock);
@@ -504,9 +516,9 @@ void voutlet_dspepilog(struct _voutlet *x, t_signal **parentsigs,
         if (bufsize < myvecsize) bufsize = myvecsize;
         if (bufsize != (oldbufsize = x->x_bufsize))
         {
-            t_float *buf = x->x_buf;
+            t_sample *buf = x->x_buf;
             t_freebytes(buf, oldbufsize * sizeof(*buf));
-            buf = (t_float *)t_getbytes(bufsize * sizeof(*buf));
+            buf = (t_sample *)t_getbytes(bufsize * sizeof(*buf));
             memset((char *)buf, 0, bufsize * sizeof(*buf));
             x->x_bufsize = bufsize;
             x->x_endbuf = buf + bufsize;
@@ -529,9 +541,11 @@ void voutlet_dspepilog(struct _voutlet *x, t_signal **parentsigs,
                     re_parentvecsize);
             else
             {
+                int method = (x->x_updown.method == 3?
+                    (pd_compatibilitylevel < 044 ? 0 : 1) : x->x_updown.method);
                 dsp_add(voutlet_doepilog_resampling, 2, x, re_parentvecsize);
                 resampleto_dsp(&x->x_updown, outsig->s_vec, re_parentvecsize,
-                    parentvecsize, x->x_updown.method);
+                    parentvecsize, method);
             }
         }
     }
@@ -556,7 +570,7 @@ static void *voutlet_newsig(t_symbol *s)
     x->x_parentoutlet = canvas_addoutlet(x->x_canvas,
         &x->x_obj.ob_pd, &s_signal);
     inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
-    x->x_endbuf = x->x_buf = (t_float *)getbytes(0);
+    x->x_endbuf = x->x_buf = (t_sample *)getbytes(0);
     x->x_bufsize = 0;
 
     resample_init(&x->x_updown);
@@ -570,7 +584,8 @@ static void *voutlet_newsig(t_symbol *s)
     if (s == gensym("hold"))x->x_updown.method=1;        /* up: sample and hold */
     else if (s == gensym("lin"))x->x_updown.method=2;    /* up: linear interpolation */
     else if (s == gensym("linear"))x->x_updown.method=2; /* up: linear interpolation */
-    else x->x_updown.method=0;                           /* up: zero-padding; down: ignore samples inbetween */
+    else if (s == gensym("pad"))x->x_updown.method=0;    /* up: zero pad */
+    else x->x_updown.method=3;                           /* up: zero-padding; down: ignore samples inbetween */
 
     return (x);
 }
