@@ -44,7 +44,7 @@ int glist_readscalar(t_glist *x, int natoms, t_atom *vec,
 static void canvas_readerror(int natoms, t_atom *vec, int message, 
     int nline, char *s)
 {
-    error(s);
+    error("%s", s);
     startpost("line was:");
     postatom(nline, vec + message);
     endpost();
@@ -258,8 +258,7 @@ static void glist_doread(t_glist *x, t_symbol *filename, t_symbol *format,
     else if (*format->s_name)
         error("qlist_read: unknown flag: %s", format->s_name);
     
-    if (binbuf_read_via_path(b, filename->s_name,
-        canvas_getdir(canvas)->s_name, cr))
+    if (binbuf_read_via_canvas(b, filename->s_name, canvas, cr))
     {
         pd_error(x, "read failed");
         binbuf_free(b);
@@ -620,15 +619,15 @@ static void canvas_saveto(t_canvas *x, t_binbuf *b)
             binbuf_addv(b, "ssfffffffff;", gensym("#X"), gensym("coords"),
                 x->gl_x1, x->gl_y1,
                 x->gl_x2, x->gl_y2,
-                (float)x->gl_pixwidth, (float)x->gl_pixheight,
-                (float)((x->gl_hidetext)?2.:1.),
-                (float)x->gl_xmargin, (float)x->gl_ymargin); 
+                (t_float)x->gl_pixwidth, (t_float)x->gl_pixheight,
+                (t_float)((x->gl_hidetext)?2.:1.),
+                (t_float)x->gl_xmargin, (t_float)x->gl_ymargin); 
                     /* otherwise write in 0.38-compatible form */
         else binbuf_addv(b, "ssfffffff;", gensym("#X"), gensym("coords"),
                 x->gl_x1, x->gl_y1,
                 x->gl_x2, x->gl_y2,
-                (float)x->gl_pixwidth, (float)x->gl_pixheight,
-                (float)x->gl_isgraph);
+                (t_float)x->gl_pixwidth, (t_float)x->gl_pixheight,
+                (t_float)x->gl_isgraph);
     }
 }
 
@@ -705,7 +704,11 @@ static void canvas_savetofile(t_canvas *x, t_symbol *filename, t_symbol *dir)
     {
             /* if not an abstraction, reset title bar and directory */ 
         if (!x->gl_owner)
+{
             canvas_rename(x, filename, dir);
+            /* update window list in case Save As changed the window name */
+            canvas_updatewindowlist(); 
+}
         post("saved to: %s/%s", dir->s_name, filename->s_name);
         canvas_dirty(x, 0);
         canvas_reload(filename, dir, &x->gl_gobj);
@@ -716,7 +719,7 @@ static void canvas_savetofile(t_canvas *x, t_symbol *filename, t_symbol *dir)
 static void canvas_menusaveas(t_canvas *x)
 {
     t_canvas *x2 = canvas_getrootfor(x);
-    sys_vgui("pdtk_canvas_saveas .x%lx \"%s\" \"%s\"\n", x2,
+    sys_vgui("pdtk_canvas_saveas .x%lx {%s} {%s}\n", x2,
         x2->gl_name->s_name, canvas_getdir(x2)->s_name);
 }
 
@@ -725,7 +728,8 @@ static void canvas_menusave(t_canvas *x)
     t_canvas *x2 = canvas_getrootfor(x);
     char *name = x2->gl_name->s_name;
     if (*name && strncmp(name, "Untitled", 8)
-            && (strlen(name) < 4 || strcmp(name + strlen(name)-4, ".pat")))
+            && (strlen(name) < 4 || strcmp(name + strlen(name)-4, ".pat")
+                || strcmp(name + strlen(name)-4, ".mxt")))
             canvas_savetofile(x2, x2->gl_name, canvas_getdir(x2));
     else canvas_menusaveas(x2);
 }
