@@ -170,6 +170,8 @@ static void canvas_howputnew(t_canvas *x, int *connectp, int *xpixp, int *ypixp,
     else
     {
         glist_getnextxy(x, xpixp, ypixp);
+        *xpixp -= 3;
+        *ypixp -= 3;
         glist_noselect(x);
     }
     *connectp = connectme;
@@ -1066,7 +1068,7 @@ static void text_activate(t_gobj *z, t_glist *glist, int state)
 static void text_delete(t_gobj *z, t_glist *glist)
 {
     t_text *x = (t_text *)z;
-    canvas_deletelinesfor(glist, x);
+        canvas_deletelinesfor(glist, x);
 }
 
 static void text_vis(t_gobj *z, t_glist *glist, int vis)
@@ -1371,10 +1373,10 @@ void text_setto(t_text *x, t_glist *glist, char *buf, int bufsize)
             int xwas = x->te_xpix, ywas = x->te_ypix;
             glist_delete(glist, &x->te_g);
             canvas_objtext(glist, xwas, ywas, 0, b);
+            canvas_restoreconnections(glist_getcanvas(glist));
                 /* if it's an abstraction loadbang it here */
             if (newest && pd_class(newest) == canvas_class)
                 canvas_loadbang((t_canvas *)newest);
-            canvas_restoreconnections(glist_getcanvas(glist));
         }
             /* if we made a new "pd" or changed a window name,
                 update window list */
