@@ -328,6 +328,7 @@ void sys_set_audio_settings(int naudioindev, int *audioindev, int nchindev,
     sys_log_error(ERR_NOTHING);
     audio_nextinchans = inchans;
     audio_nextoutchans = outchans;
+    sys_setchsr(audio_nextinchans, audio_nextoutchans, rate);
     sys_save_audio_params(nrealindev, realindev, realinchans,
         nrealoutdev, realoutdev, realoutchans, rate, advance, callback,
             blocksize);
@@ -591,6 +592,13 @@ void sys_reportidle(void)
 {
 }
 
+/* this could later be set by a preference but for now it seems OK to just
+keep jack audio open but close unused audio devices for any other API */
+int audio_shouldkeepopen( void)
+{
+    return (sys_audioapi == API_JACK);
+}
+
 static void audio_getdevs(char *indevlist, int *nindevs,
     char *outdevlist, int *noutdevs, int *canmulti, int *cancallback,
         int maxndev, int devdescsize)
@@ -909,9 +917,40 @@ void sys_get_audio_devs(char *indevlist, int *nindevs,
 
 void sys_set_audio_api(int which)
 {
-     sys_audioapi = which;
-     if (sys_verbose)
-        post("sys_audioapi %d", sys_audioapi);
+    int ok = 0;    /* check if the API is actually compiled in */
+#ifdef USEAPI_PORTAUDIO
+    ok += (which == API_PORTAUDIO);
+#endif
+#ifdef USEAPI_JACK
+    ok += (which == API_JACK);
+#endif
+#ifdef USEAPI_OSS
+    ok += (which == API_OSS);
+#endif
+#ifdef USEAPI_ALSA
+    ok += (which == API_ALSA);
+#endif
+#ifdef USEAPI_MMIO
+    ok += (which == API_MMIO);
+#endif
+#ifdef USEAPI_AUDIOUNIT
+    ok += (which == API_AUDIOUNIT);
+#endif
+#ifdef USEAPI_ESD
+    ok += (which == API_ESD);
+#endif
+#ifdef USEAPI_DUMMY
+    ok += (which == API_DUMMY);
+#endif
+    if (!ok)
+    {
+        post("API %d not supported, reverting to %d (%s)",
+            which, API_DEFAULT, API_DEFSTRING);
+        which = API_DEFAULT;
+    }
+    sys_audioapi = which;
+    if (sys_verbose && ok)
+        post("sys_audioapi set to %d", sys_audioapi);
 }
 
 void glob_audio_setapi(void *dummy, t_floatarg f)
