@@ -5,7 +5,7 @@
 
 /* this file inputs and outputs audio using the ALSA API available on linux. */
 
-/* support for ALSA pcmv2 api by Karl MacMillan<karlmac@peabody.jhu.edu> */ 
+/* support for ALSA pcmv2 api by Karl MacMillan<karlmac@peabody.jhu.edu> */
 /* support for ALSA MMAP noninterleaved by Winfried Ritsch, IEM */
 
 #include <alsa/asoundlib.h>
@@ -41,7 +41,7 @@ static void alsa_numbertoname(int iodev, char *devname, int nchar);
 static int alsa_jittermax;
 #define ALSA_DEFJITTERMAX 3
 
-    /* don't assume we can turn all 31 bits when doing float-to-fix; 
+    /* don't assume we can turn all 31 bits when doing float-to-fix;
     otherwise some audio drivers (e.g. Midiman/ALSA) wrap around. */
 #define FMAX 0x7ffff000
 #define CLIP32(x) (((x)>FMAX)?FMAX:((x) < -FMAX)?-FMAX:(x))
@@ -49,7 +49,7 @@ static int alsa_jittermax;
 static char *alsa_snd_buf;
 static int alsa_snd_bufsize;
 static int alsa_buf_samps;
-static snd_pcm_status_t *alsa_status; 
+static snd_pcm_status_t *alsa_status;
 static int alsa_usemmap;
 
 t_alsa_dev alsa_indev[ALSA_MAXDEV];
@@ -57,22 +57,36 @@ t_alsa_dev alsa_outdev[ALSA_MAXDEV];
 int alsa_nindev;
 int alsa_noutdev;
 
-static void check_error(int err, const char *why)
+/* #define DEBUG_ALSA_XFER */
+#ifdef DEBUG_ALSA_XFER
+static int xferno = 0;
+static int callno = 0;
+#endif
+
+/* report an error condition if an error was flagged in the argument "err".
+"fn" is 0 for input, 1 for output, otherwise N/A.  "why" indicates where
+in the code the error was hit. */
+static void check_error(int err, int fn, const char *why)
 {
     if (err < 0)
-        fprintf(stderr, "%s: %s\n", why, snd_strerror(err));
+        post("ALSA %serror (%s): %s",
+            (fn == 1? "output " : (fn == 0 ? "input ": "")),
+                why, snd_strerror(err));
 }
 
+/* figure out, when opening ALSA device, whether we should use the code in
+this file or defer to Winfried Ritch's code to do mmaped transfers (handled
+in s_audio_alsamm.c). */
 static int alsaio_canmmap(t_alsa_dev *dev)
 {
     snd_pcm_hw_params_t *hw_params;
     int err1, err2;
-    
+
     snd_pcm_hw_params_alloca(&hw_params);
 
     err1 = snd_pcm_hw_params_any(dev->a_handle, hw_params);
     if (err1 < 0) {
-      check_error(err1,"Broken configuration: no configurations available"); 
+      check_error(err1, -1, "snd_pcm_hw_params_any");
       return (0);
     }
     err1 = snd_pcm_hw_params_set_access(dev->a_handle,
@@ -90,42 +104,35 @@ static int alsaio_canmmap(t_alsa_dev *dev)
     return ((err1 < 0) && (err2 >= 0));
 }
 
-static void check_setup_error(int err, int out, const char *why) {
-        char bf[256];
-        snprintf(bf, sizeof bf, "%s (%s)", why, out ? "output" : "input");
-        check_error(err, bf);
-}
-
+/* set up an input or output device.  Return 0 on success, -1 on failure. */
 static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
     int nfrags, int frag_size)
 {
-#define CHECK_ERROR(why_) check_setup_error(err, out, why_)
-
     int bufsizeforthis, err;
     snd_pcm_hw_params_t* hw_params;
+    snd_pcm_sw_params_t* sw_params;
     unsigned int tmp_uint;
     snd_pcm_uframes_t tmp_snd_pcm_uframes;
 
+    snd_pcm_hw_params_alloca(&hw_params);
+    snd_pcm_sw_params_alloca(&sw_params);
+
     if (sys_verbose)
-    {
-        if (out)
-            post("configuring sound output...");
-        else post("configuring sound input...");
-    }
+        post((out ? "configuring sound output..." :
+            "configuring sound input..."));
 
         /* set hardware parameters... */
-    snd_pcm_hw_params_alloca(&hw_params);
 
         /* get the default params */
     err = snd_pcm_hw_params_any(dev->a_handle, hw_params);
-    CHECK_ERROR("snd_pcm_hw_params_any");
+    check_error(err, out, "snd_pcm_hw_params_any");
 
         /* try to set interleaved access */
     err = snd_pcm_hw_params_set_access(dev->a_handle,
         hw_params, SND_PCM_ACCESS_RW_INTERLEAVED);
     if (err < 0)
         return (-1);
-    CHECK_ERROR("snd_pcm_hw_params_set_access");
+    check_error(err, out, "snd_pcm_hw_params_set_access");
 #if 0       /* enable this to print out which formats are available */
     {
         int i;
@@ -139,17 +146,13 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
         hw_params, SND_PCM_FORMAT_S32);
     if (err < 0)
     {
-        /* fprintf(stderr,
-            "PD-ALSA: 32 bit format not available - trying 24\n"); */
         err = snd_pcm_hw_params_set_format(dev->a_handle, hw_params,
             SND_PCM_FORMAT_S24_3LE);
         if (err < 0)
         {
-            /* fprintf(stderr,
-                "PD-ALSA: 32/24 bit format not available - using 16\n"); */
             err = snd_pcm_hw_params_set_format(dev->a_handle, hw_params,
                 SND_PCM_FORMAT_S16);
-            CHECK_ERROR("_pcm_hw_params_set_format");
+            check_error(err, out, "_pcm_hw_params_set_format");
             dev->a_sampwidth = 2;
         }
         else dev->a_sampwidth = 3;
@@ -162,22 +165,22 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
         /* set the subformat */
     err = snd_pcm_hw_params_set_subformat(dev->a_handle,
         hw_params, SND_PCM_SUBFORMAT_STD);
-    CHECK_ERROR("snd_pcm_hw_params_set_subformat");
-    
+    check_error(err, out, "snd_pcm_hw_params_set_subformat");
+
         /* set the number of channels */
     tmp_uint = *channels;
     err = snd_pcm_hw_params_set_channels_min(dev->a_handle,
         hw_params, &tmp_uint);
-    CHECK_ERROR("snd_pcm_hw_params_set_channels");
+    check_error(err, out, "snd_pcm_hw_params_set_channels");
     if (tmp_uint != (unsigned)*channels)
         post("ALSA: set %s channels to %d", (out?"output":"input"), tmp_uint);
     *channels = tmp_uint;
     dev->a_channels = *channels;
 
         /* set the sampling rate */
-    err = snd_pcm_hw_params_set_rate_near(dev->a_handle, hw_params, 
+    err = snd_pcm_hw_params_set_rate_near(dev->a_handle, hw_params,
         (unsigned int *)rate, 0);
-    CHECK_ERROR("snd_pcm_hw_params_set_rate_min");
+    check_error(err, out, "snd_pcm_hw_params_set_rate_min");
 #if 0
     err = snd_pcm_hw_params_get_rate(hw_params, &subunitdir);
     post("input sample rate %d", err);
@@ -185,29 +188,19 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
 
     /* post("frag size %d, nfrags %d", frag_size, nfrags); */
         /* set "period size" */
-#ifdef ALSAAPI9
-    err = snd_pcm_hw_params_set_period_size_near(dev->a_handle,
-        hw_params, (snd_pcm_uframes_t)frag_size, 0);
-#else
     tmp_snd_pcm_uframes = frag_size;
     err = snd_pcm_hw_params_set_period_size_near(dev->a_handle,
         hw_params, &tmp_snd_pcm_uframes, 0);
-#endif
-    CHECK_ERROR("snd_pcm_hw_params_set_period_size_near");
+    check_error(err, out, "snd_pcm_hw_params_set_period_size_near");
 
         /* set the buffer size */
-#ifdef ALSAAPI9
-    err = snd_pcm_hw_params_set_buffer_size_near(dev->a_handle,
-        hw_params, nfrags * frag_size);
-#else
     tmp_snd_pcm_uframes = nfrags * frag_size;
     err = snd_pcm_hw_params_set_buffer_size_near(dev->a_handle,
         hw_params, &tmp_snd_pcm_uframes);
-#endif
-    CHECK_ERROR("snd_pcm_hw_params_set_buffer_size_near");
+    check_error(err, out, "snd_pcm_hw_params_set_buffer_size_near");
 
     err = snd_pcm_hw_params(dev->a_handle, hw_params);
-    CHECK_ERROR("snd_pcm_hw_params");
+    check_error(err, out, "snd_pcm_hw_params");
 
         /* set up the buffer */
     bufsizeforthis = DEFDACBLKSIZE * dev->a_sampwidth * *channels;
@@ -218,7 +211,7 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
             if (!(alsa_snd_buf = realloc(alsa_snd_buf, bufsizeforthis)))
             {
                 post("out of memory");
-                return (0);
+                return (-1);
             }
             memset(alsa_snd_buf, 0, bufsizeforthis);
             alsa_snd_bufsize = bufsizeforthis;
@@ -229,13 +222,44 @@ static int alsaio_setup(t_alsa_dev *dev, int out, int *channels, int *rate,
         if (!(alsa_snd_buf = (void *)malloc(bufsizeforthis)))
         {
             post("out of memory");
-            return (0);
+            return (-1);
         }
         memset(alsa_snd_buf, 0, bufsizeforthis);
         alsa_snd_bufsize = bufsizeforthis;
     }
-    return (1);
-#undef CHECK_ERROR
+    
+    err = snd_pcm_sw_params_current(dev->a_handle, sw_params);
+    if (err < 0)
+    {
+        post("Unable to determine current swparams for %s: %s\n",
+            (out ? "output" : "input"), snd_strerror(err));
+        return (-1);
+    }
+    err = snd_pcm_sw_params_set_stop_threshold(dev->a_handle, sw_params,
+        0x7fffffff);
+    if (err < 0)
+    {
+        post("Unable to set start threshold mode for %s: %s\n",
+            (out ? "output" : "input"), snd_strerror(err));
+        return (-1);
+    }
+    
+    err = snd_pcm_sw_params_set_avail_min(dev->a_handle, sw_params, 4);
+    if (err < 0)
+    {
+        post("Unable to set avail min for %s: %s\n",
+            (out ? "output" : "input"), snd_strerror(err));
+        return (-1);
+    }
+    err = snd_pcm_sw_params(dev->a_handle, sw_params);
+    if (err < 0)
+    {
+        post("Unable to set sw params for %s: %s\n",
+            (out ? "output" : "input"), snd_strerror(err));
+        return (-1);
+    }
+
+    return (0);
 }
 
 
@@ -265,7 +289,7 @@ int alsa_open_audio(int naudioindev, int *audioindev, int nchindev,
         alsa_numbertoname(audioindev[iodev], devname, 512);
         err = snd_pcm_open(&alsa_indev[alsa_nindev].a_handle, devname,
             SND_PCM_STREAM_CAPTURE, SND_PCM_NONBLOCK);
-        check_error(err, "snd_pcm_open (input)");
+        check_error(err, 0, "snd_pcm_open");
         if (err < 0)
             continue;
         alsa_indev[alsa_nindev].a_devno = audioindev[iodev];
@@ -279,7 +303,7 @@ int alsa_open_audio(int naudioindev, int *audioindev, int nchindev,
         alsa_numbertoname(audiooutdev[iodev], devname, 512);
         err = snd_pcm_open(&alsa_outdev[alsa_noutdev].a_handle, devname,
             SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK);
-        check_error(err, "snd_pcm_open (output)");
+        check_error(err, 1, "snd_pcm_open");
         if (err < 0)
             continue;
         alsa_outdev[alsa_noutdev].a_devno = audiooutdev[iodev];
@@ -344,10 +368,11 @@ int alsa_open_audio(int naudioindev, int *audioindev, int nchindev,
     if (!alsa_status)
     {
         err = snd_pcm_status_malloc(&alsa_status);
-        check_error(err, "snd_pcm_status_malloc");
+        check_error(err, -1, "snd_pcm_status_malloc");
     }
 
-        /* fill the buffer with silence */
+        /* fill the buffer with silence and prime the output FIFOs.  This
+        should automatically start the output devices. */
     memset(alsa_snd_buf, 0, alsa_snd_bufsize);
 
     if (outchans)
@@ -360,11 +385,15 @@ int alsa_open_audio(int naudioindev, int *audioindev, int nchindev,
                     DEFDACBLKSIZE);
         }
     }
-    else if (inchans)
+    if (inchans)
     {
+            /* some of the ADC devices might already have been started by
+            starting the outputs above, but others might still need it. */
         for (iodev = 0; iodev < alsa_nindev; iodev++)
-            if ((err = snd_pcm_start(alsa_indev[iodev].a_handle)) < 0)
-                check_error(err, "input start failed\n");
+            if (snd_pcm_state(alsa_indev[iodev].a_handle)
+                != SND_PCM_STATE_RUNNING)
+                    if ((err = snd_pcm_start(alsa_indev[iodev].a_handle)) < 0)
+                        check_error(err, -1, "input start failed");
     }
     return (0);
 blewit:
@@ -385,26 +414,22 @@ void alsa_close_audio(void)
     for (iodev = 0; iodev < alsa_nindev; iodev++)
     {
         err = snd_pcm_close(alsa_indev[iodev].a_handle);
-        check_error(err, "snd_pcm_close (input)");
+        check_error(err, 0, "snd_pcm_close");
     }
     for (iodev = 0; iodev < alsa_noutdev; iodev++)
     {
         err = snd_pcm_close(alsa_outdev[iodev].a_handle);
-        check_error(err, "snd_pcm_close (output)");
+        check_error(err, 1, "snd_pcm_close");
     }
     alsa_nindev = alsa_noutdev = 0;
 }
 
 int alsa_send_dacs(void)
 {
-#ifdef DEBUG_ALSA_XFER
-    static int xferno = 0;
-    static int callno = 0;
-#endif
     static double timenow;
     double timelast;
     t_sample *fp, *fp1, *fp2;
-    int i, j, k, err, iodev, result, ch; 
+    int i, j, k, err, iodev, result, ch, resync = 0;;
     int chansintogo, chansouttogo;
     unsigned int transfersize;
 
@@ -423,25 +448,44 @@ int alsa_send_dacs(void)
 
 #ifdef DEBUG_ALSA_XFER
     if (timenow - timelast > 0.050)
-        fprintf(stderr, "(%d)",
+        post("long wait between calls: %d",
             (int)(1000 * (timenow - timelast))), fflush(stderr);
     callno++;
 #endif
 
-    alsa_checkiosync();     /* check I/O are in sync and data not late */
-
+   
     for (iodev = 0; iodev < alsa_nindev; iodev++)
     {
+        result = snd_pcm_state(alsa_indev[iodev].a_handle);
+        if (result == SND_PCM_STATE_XRUN)
+        {
+            int res2 = snd_pcm_start(alsa_indev[iodev].a_handle);
+            fprintf(stderr, "restart alsa input\n");
+            if (res2 < 0)
+                fprintf(stderr, "alsa xrun recovery apparently failed\n");
+        }
         snd_pcm_status(alsa_indev[iodev].a_handle, alsa_status);
         if (snd_pcm_status_get_avail(alsa_status) < transfersize)
-            return SENDDACS_NO;
+            return (SENDDACS_NO);
     }
     for (iodev = 0; iodev < alsa_noutdev; iodev++)
     {
+        result = snd_pcm_state(alsa_outdev[iodev].a_handle);
+        if (result == SND_PCM_STATE_XRUN)
+        {
+            int res2 = snd_pcm_start(alsa_outdev[iodev].a_handle);
+            fprintf(stderr, "restart alsa output\n");
+            if (res2 < 0)
+                fprintf(stderr, "alsa xrun recovery apparently failed\n");
+        }
         snd_pcm_status(alsa_outdev[iodev].a_handle, alsa_status);
         if (snd_pcm_status_get_avail(alsa_status) < transfersize)
-            return SENDDACS_NO;
+            return (SENDDACS_NO);
     }
+
+#ifdef DEBUG_ALSA_XFER
+    post("xfer %d", transfersize);
+#endif
     /* do output */
     for (iodev = 0, fp1 = sys_soundout, ch = 0; iodev < alsa_noutdev; iodev++)
     {
@@ -457,7 +501,7 @@ int alsa_send_dacs(void)
             {
                 float s1 = *fp2 * INT32_MAX;
                 ((t_alsa_sample32 *)alsa_snd_buf)[j] = CLIP32(s1);
-            } 
+            }
             for (; i < thisdevchans; i++, ch++)
                 for (j = i, k = DEFDACBLKSIZE; k--; j += thisdevchans)
                     ((t_alsa_sample32 *)alsa_snd_buf)[j] = 0;
@@ -473,7 +517,7 @@ int alsa_send_dacs(void)
                     s = 8388351;
                 else if (s < -8388351)
                     s = -8388351;
-#if BYTE_ORDER == LITTLE_ENDIAN                                             
+#if BYTE_ORDER == LITTLE_ENDIAN
                 ((char *)(alsa_snd_buf))[3*j] = (s & 255);
                 ((char *)(alsa_snd_buf))[3*j+1] = ((s>>8) & 255);
                 ((char *)(alsa_snd_buf))[3*j+2] = ((s>>16) & 255);
@@ -483,8 +527,8 @@ int alsa_send_dacs(void)
             }
             for (; i < thisdevchans; i++, ch++)
                 for (j = i, k = DEFDACBLKSIZE; k--; j += thisdevchans)
-                    ((char *)(alsa_snd_buf))[3*j] = 
-                    ((char *)(alsa_snd_buf))[3*j+1] = 
+                    ((char *)(alsa_snd_buf))[3*j] =
+                    ((char *)(alsa_snd_buf))[3*j+1] =
                     ((char *)(alsa_snd_buf))[3*j+2] = 0;
         }
         else        /* 16 bit samples */
@@ -511,18 +555,20 @@ int alsa_send_dacs(void)
         {
     #ifdef DEBUG_ALSA_XFER
             if (result >= 0 || errno == EAGAIN)
-                fprintf(stderr, "ALSA: write returned %d of %d\n",
+                post("ALSA: write returned %d of %d\n",
                         result, transfersize);
-            else fprintf(stderr, "ALSA: write: %s\n",
+            else post("ALSA: write: %s\n",
                          snd_strerror(errno));
-            fprintf(stderr,
-                    "inputcount %d, outputcount %d, outbufsize %d\n",
-                    inputcount, outputcount, 
-                    (ALSA_EXTRABUFFER + sys_advance_samples)
-                    * alsa_outdev[iodev].a_sampwidth * outchannels);
     #endif
-            sys_log_error(ERR_DACSLEPT);
-            return (SENDDACS_NO);
+            sys_log_error(ERR_DATALATE);
+            if (result == -EPIPE)
+            {
+                result = snd_pcm_prepare(alsa_indev[iodev].a_handle);
+                if (result < 0)
+                    fprintf(stderr, "read reset error %d\n", result);
+            }
+            else fprintf(stderr, "read other error %d\n", result);
+            resync = 1;
         }
 
         /* zero out the output buffer */
@@ -531,7 +577,7 @@ int alsa_send_dacs(void)
         if (sys_getrealtime() - timenow > 0.002)
         {
     #ifdef DEBUG_ALSA_XFER
-            fprintf(stderr, "output %d took %d msec\n",
+            post("output %d took %d msec\n",
                     callno, (int)(1000 * (timenow - timelast))), fflush(stderr);
     #endif
             timenow = sys_getrealtime();
@@ -551,20 +597,20 @@ int alsa_send_dacs(void)
         {
 #ifdef DEBUG_ALSA_XFER
             if (result < 0)
-                fprintf(stderr,
-                        "snd_pcm_read %d %d: %s\n", 
+                post("snd_pcm_read %d %d: %s\n",
                         callno, xferno, snd_strerror(errno));
-            else fprintf(stderr,
-                         "snd_pcm_read %d %d returned only %d\n", 
+            else post("snd_pcm_read %d %d returned only %d\n",
                          callno, xferno, result);
-            fprintf(stderr,
-                    "inputcount %d, outputcount %d, inbufsize %d\n",
-                    inputcount, outputcount, 
-                    (ALSA_EXTRABUFFER + sys_advance_samples)
-                    * alsa_indev[iodev].a_sampwidth * inchannels);
 #endif
-            sys_log_error(ERR_ADCSLEPT);
-            return (SENDDACS_NO);
+            sys_log_error(ERR_DATALATE);
+            if (result == -EPIPE)
+            {
+                result = snd_pcm_prepare(alsa_indev[iodev].a_handle);
+                if (result < 0)
+                    fprintf(stderr, "read reset error %d\n", result);
+            }
+            else fprintf(stderr, "read other error %d\n", result);
+            resync = 1;
         }
         if (alsa_indev[iodev].a_sampwidth == 4)
         {
@@ -578,7 +624,7 @@ int alsa_send_dacs(void)
         }
         else if (alsa_indev[iodev].a_sampwidth == 3)
         {
-#if BYTE_ORDER == LITTLE_ENDIAN                                             
+#if BYTE_ORDER == LITTLE_ENDIAN
             for (i = 0; i < chans; i++, ch++, fp1 += DEFDACBLKSIZE)
             {
                 for (j = ch, k = DEFDACBLKSIZE, fp2 = fp1; k--;
@@ -610,11 +656,20 @@ int alsa_send_dacs(void)
     if (sys_getrealtime() - timenow > 0.002)
     {
 #ifdef DEBUG_ALSA_XFER
-        fprintf(stderr, "routine took %d msec\n",
+        post("alsa_send_dacs took %d msec\n",
             (int)(1000 * (sys_getrealtime() - timenow)));
 #endif
         sys_log_error(ERR_ADCSLEPT);
     }
+    {
+        static int checkcountdown = 0;
+        if (!checkcountdown--)
+        {
+            checkcountdown = 10;
+            if (alsa_nindev + alsa_noutdev > 1)
+                alsa_checkiosync();   /*  check I/O are in sync */     
+        }
+    }
     return SENDDACS_YES;
 }
 
@@ -646,28 +701,6 @@ void alsa_printstate( void)
     post("buf samples %d", alsa_buf_samps);
 }
 
-
-void alsa_resync( void)
-{
-    int i, result, iodev = 0;
-    if (sys_audioapi != API_ALSA)
-    {
-        error("restart-audio: implemented for ALSA only.");
-        return;
-    }
-    memset(alsa_snd_buf, 0,
-        alsa_indev[iodev].a_sampwidth *
-            DEFDACBLKSIZE * sys_outchannels);
-    for (i = 0; i < 1000000; i++)
-    {
-        result = snd_pcm_writei(alsa_outdev[iodev].a_handle, alsa_snd_buf,
-            DEFDACBLKSIZE);
-        if (result != (int)DEFDACBLKSIZE)
-            break;
-    }
-    post("%d written", i);
-}
-
 void alsa_putzeros(int iodev, int n)
 {
     int i, result;
@@ -704,7 +737,7 @@ void alsa_getzeros(int iodev, int n)
     /* call this only if both input and output are open */
 static void alsa_checkiosync( void)
 {
-    int i, result, giveup = 1000, alreadylogged = 0, iodev = 0;
+    int i, result, giveup = 50, alreadylogged = 0, iodev = 0, err;
     snd_pcm_sframes_t minphase, maxphase, thisphase, outdelay;
 
     while (1)
@@ -719,12 +752,25 @@ static void alsa_checkiosync( void)
         maxphase = -0x7fffffff;
         for (iodev = 0; iodev < alsa_noutdev; iodev++)
         {
+            if ((result = snd_pcm_state(alsa_outdev[iodev].a_handle))
+                != SND_PCM_STATE_RUNNING && result != SND_PCM_STATE_XRUN)
+            {
+                if (sys_verbose)
+                    post("restarting output device from state %d",
+                        snd_pcm_state(alsa_outdev[iodev].a_handle));
+                if ((err = snd_pcm_start(alsa_outdev[iodev].a_handle)) < 0)
+                    check_error(err, 0, "restart failed");
+            }
             result = snd_pcm_delay(alsa_outdev[iodev].a_handle, &outdelay);
             if (result < 0)
             {
                 snd_pcm_prepare(alsa_outdev[iodev].a_handle);
                 result = snd_pcm_delay(alsa_outdev[iodev].a_handle, &outdelay);
             }
+#ifdef DEBUG_ALSA_XFER
+            post("outfifo %d %d %d",
+                callno, xferno, outdelay);
+#endif
             if (result < 0)
             {
                 post("output snd_pcm_delay failed: %s", snd_strerror(result));
@@ -745,12 +791,25 @@ static void alsa_checkiosync( void)
         }
         for (iodev = 0; iodev < alsa_nindev; iodev++)
         {
+            if ((result = snd_pcm_state(alsa_indev[iodev].a_handle))
+                != SND_PCM_STATE_RUNNING && result != SND_PCM_STATE_XRUN)
+            {
+                if (sys_verbose)
+                    post("restarting input device from state %d",
+                        snd_pcm_state(alsa_indev[iodev].a_handle));
+                if ((err = snd_pcm_start(alsa_indev[iodev].a_handle)) < 0)
+                    check_error(err, 1, "restart failed");
+            }
             result = snd_pcm_delay(alsa_indev[iodev].a_handle, &thisphase);
             if (result < 0)
             {
                 snd_pcm_prepare(alsa_indev[iodev].a_handle);
                 result = snd_pcm_delay(alsa_indev[iodev].a_handle, &thisphase);
             }
+#ifdef DEBUG_ALSA_XFER
+            post("infifo  %d %d %d",
+                callno, xferno, thisphase);
+#endif
             if (result < 0)
             {
                 post("output snd_pcm_delay failed: %s", snd_strerror(result));
@@ -773,8 +832,10 @@ static void alsa_checkiosync( void)
         if (maxphase <= minphase + (alsa_jittermax * (DEFDACBLKSIZE / 4)))
                 break;
 
-        if (!alreadylogged)
-            sys_log_error(ERR_RESYNC), alreadylogged = 1;
+#ifdef DEBUG_ALSA_XFER
+            post("resync audio %d %d %d", xferno, minphase, maxphase);
+#endif
+        
         for (iodev = 0; iodev < alsa_noutdev; iodev++)
         {
             result = snd_pcm_delay(alsa_outdev[iodev].a_handle, &outdelay);
@@ -784,8 +845,11 @@ static void alsa_checkiosync( void)
             if (thisphase > minphase + DEFDACBLKSIZE)
             {
                 alsa_putzeros(iodev, 1);
-#if DEBUGSYNC
-                fprintf(stderr, "putz %d %d\n", (int)thisphase, (int)minphase);
+                if (!alreadylogged)
+                    sys_log_error(ERR_RESYNC), alreadylogged = 1;
+#ifdef DEBUG_ALSA_XFER
+                post("putz %d %d %d %d",
+                    callno, xferno, (int)thisphase, (int)minphase);
 #endif
             }
         }
@@ -797,15 +861,21 @@ static void alsa_checkiosync( void)
             if (thisphase > minphase + DEFDACBLKSIZE)
             {
                 alsa_getzeros(iodev, 1);
-#if DEBUGSYNC
-                fprintf(stderr, "getz %d %d\n", (int)thisphase, (int)minphase);
+                if (!alreadylogged)
+                    sys_log_error(ERR_RESYNC), alreadylogged = 1;
+#ifdef DEBUG_ALSA_XFER
+                post("getz %d %d %d %d",
+                    callno, xferno, (int)thisphase, (int)minphase);
 #endif
             }
         }
+            /* it's possible we didn't do anything; if so don't repeat */
+        if (!alreadylogged)
+            break;
     }
-#if DEBUGSYNC
+#ifdef DEBUG_ALSA_XFER
     if (alreadylogged)
-        fprintf(stderr, "done\n");
+        post("done resync");
 #endif
 }
 
@@ -842,7 +912,7 @@ static void alsa_numbertoname(int devno, char *devname, int nchar)
     /* For each hardware card found, we list two devices, the "hard" and
     "plug" one.  The card scan is derived from portaudio code. */
 void alsa_getdevs(char *indevlist, int *nindevs,
-    char *outdevlist, int *noutdevs, int *canmulti, 
+    char *outdevlist, int *noutdevs, int *canmulti,
         int maxndev, int devdescsize)
 {
     int ndev = 0, cardno = -1, i, j;
@@ -883,7 +953,9 @@ void alsa_getdevs(char *indevlist, int *nindevs,
     {
         if (j >= maxndev)
             break;
-        snprintf(indevlist + j * devdescsize, devdescsize, "%s", 
+        snprintf(indevlist + j * devdescsize, devdescsize, "%s",
+            alsa_names[i]);
+        snprintf(outdevlist + j * devdescsize, devdescsize, "%s",
             alsa_names[i]);
     }
     *nindevs = *noutdevs = j;
