@@ -19,6 +19,8 @@ that didn't really belong anywhere. */
 #include <sys/time.h>
 #include <sys/mman.h>
 #include <sys/resource.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #endif
 #ifdef HAVE_BSTRING_H
 #include <bstring.h>
@@ -769,10 +771,8 @@ static int sys_poll_togui(void) /* returns 1 if did anything */
 {
     if (sys_nogui)
         return (0);
-        /* see if there is stuff still in the buffer, if so we
-            must have fallen behind, so just try to clear that. */
-    if (sys_flushtogui())
-        return (1);
+        /* in case there is stuff still in the buffer, try to flush it. */
+    sys_flushtogui();
         /* if the flush wasn't complete, wait. */
     if (sys_guibufhead > sys_guibuftail)
         return (0);
@@ -1114,6 +1114,7 @@ int sys_startgui(const char *libdir)
 #endif /* NOT __APPLE__ */
             execl("/bin/sh", "sh", "-c", sys_guicmd, (char*)0);
             perror("pd: exec");
+            fprintf(stderr, "Perhaps tcl and tk aren't yet installed?\n");
             _exit(1);
        }
 #else /* NOT _WIN32 */
@@ -1154,7 +1155,20 @@ int sys_startgui(const char *libdir)
         /etc/security/limits.conf */
     if (sys_hipriority == -1)
         sys_hipriority = 1;
-    
+
+    sprintf(cmdbuf, "%s/bin/pd-watchdog", libdir);
+    if (sys_hipriority)
+    {
+        struct stat statbuf;
+        if (stat(cmdbuf, &statbuf) < 0)
+        {
+            fprintf(stderr,
+              "disabling real-time priority due to missing pd-watchdog (%s)\n",
+                cmdbuf);
+            sys_hipriority = 0;
+        }
+    }
+
     if (sys_hipriority)
     {
             /* To prevent lockup, we fork off a watchdog process with
@@ -1194,8 +1208,7 @@ int sys_startgui(const char *libdir)
             }
             close(pipe9[1]);
 
-            sprintf(cmdbuf, "%s/bin/pd-watchdog\n", libdir);
-            if (sys_verbose) fprintf(stderr, "%s", cmdbuf);
+            if (sys_verbose) fprintf(stderr, "%s\n", cmdbuf);
             execl("/bin/sh", "sh", "-c", cmdbuf, (char*)0);
             perror("pd: exec");
             _exit(1);
