@@ -195,41 +195,45 @@ void sys_setalarm(int microsec);
 #define API_MMIO 3
 #define API_PORTAUDIO 4
 #define API_JACK 5
-#define API_SGI 6
+#define API_SGI 6           /* gone */
 #define API_AUDIOUNIT 7
-#define API_ESD 8
+#define API_ESD 8           /* no idea what this was, probably gone now */
 #define API_DUMMY 9
 
-#ifdef USEAPI_DUMMY
-#define API_DEFAULT API_DUMMY
-#define API_DEFSTRING "dummy audio"
-#else
-#if defined(__linux__) || defined(__FreeBSD_kernel__)
-# define API_DEFAULT API_OSS
-# define API_DEFSTRING "OSS"
-#endif
-#if defined(_WIN32) || defined(__CYGWIN__)
+    /* figure out which API should be the default.  The one we judge most
+    likely to offer a working device takes precedence so that if you
+    start up Pd for the first time there's a reasonable chance you'll have
+    sound.  (You'd think portaudio would be best but it seems to default
+    to jack on linux, and and on Windows we only use it for ASIO). 
+    If nobody shows up, define DUMMY and make it the default.*/
+#if defined(USEAPI_MMIO)
 # define API_DEFAULT API_MMIO
 # define API_DEFSTRING "MMIO"
-#endif
-#ifdef __APPLE__
-# ifdef __arm__
-#  define API_DEFAULT API_AUDIOUNIT
-#  define API_DEFSTRING "AudioUnit"
-# else
+#elif defined(USEAPI_ALSA)
+# define API_DEFAULT API_ALSA
+# define API_DEFSTRING "ALSA"
+#elif defined(USEAPI_OSS)
+# define API_DEFAULT API_OSS
+# define API_DEFSTRING "OSS"
+#elif defined(USEAPI_AUDIOUNIT)
+# define API_DEFAULT API_AUDIOUNIT
+# define API_DEFSTRING "AudioUnit"
+#elif defined(USEAPI_ESD)
+# define API_DEFAULT API_ESD
+# define API_DEFSTRING "ESD (?)"
+#elif defined(USEAPI_PORTAUDIO)
 # define API_DEFAULT API_PORTAUDIO
 # define API_DEFSTRING "portaudio"
-# endif /* __arm__ */
-#endif
-#ifdef IRIX
-# define API_DEFAULT API_SGI
-# define API_DEFSTRING "SGI Digital Media"
-#endif
-#ifdef __GNU__
+#elif defined(USEAPI_JACK)
 # define API_DEFAULT API_JACK
 # define API_DEFSTRING "Jack audio connection kit"
-#endif
-#endif
+#else 
+# ifndef USEAPI_DUMMY   /* we need at least one so bring in the dummy */
+# define USEAPI_DUMMY
+# endif /* USEAPI_DUMMY */
+# define API_DEFAULT API_DUMMY
+# define API_DEFSTRING "dummy audio"
+#endif 
 
 #define DEFAULTAUDIODEV 0
 
